@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,28 +19,21 @@
 sabnzbd.codecs - Unicoded filename support
 """
 
-__NAME__ = "codecs"
-
-import os
-import sys
 import locale
 from xml.sax.saxutils import escape
 from Cheetah.Filters import Filter
 
 import sabnzbd
 
-
-gNT = os.name == 'nt'
 try:
     gUTF = locale.getdefaultlocale()[1].lower().find('utf') >= 0
 except:
     # Incorrect locale implementation, assume the worst
     gUTF = False
 
-
 def reliable_unpack_names():
     """ See if it is safe to rely on unrar names """
-    if gNT or sabnzbd.DARWIN:
+    if sabnzbd.WIN32 or sabnzbd.DARWIN:
         return True
     else:
         return gUTF
@@ -48,29 +41,70 @@ def reliable_unpack_names():
 def name_fixer(p):
     """ Return UTF-8 encoded string, if appropriate for the platform """
 
-    if sabnzbd.DARWIN:
+    if sabnzbd.DARWIN and p:
         return p.decode('Latin-1', 'replace').encode('utf-8', 'replace').replace('?', '_')
     else:
         return p
 
+def special_fixer(p):
+    """ Return string appropriate for the platform.
+        Also takes care of the situation where a non-Windows/UTF-8 system
+        receives a latin-1 encoded name.
+    """
+    if sabnzbd.WIN32:
+        try:
+            return p.decode('utf-8').encode('latin-1')
+        except:
+            return p
+    else:
+        if gUTF or sabnzbd.DARWIN:
+            try:
+                # First see if it isn't just UTF-8
+                p.decode('utf-8')
+                return p
+            except:
+                # Now assume it's latin-1
+                return p.decode('Latin-1').encode('utf-8')
+        else:
+            return p
+
+def unicoder(p):
+    """ Make sure a Unicode string is returned """
+    if isinstance(p, unicode):
+        return p
+    if isinstance(p, str):
+        if sabnzbd.DARWIN or gUTF:
+            try:
+                return p.decode('utf-8')
+            except:
+                return p.decode('latin-1', 'replace')
+        return p.decode('latin-1', 'replace')
+    else:
+        return unicode(str(p))
 
 def unicode2local(p):
     """ Convert Unicode filename to appropriate local encoding
     """
-    if gNT:
+    if sabnzbd.WIN32:
         return p.encode('Latin-1', 'replace').replace('?', '_')
     else:
         return p.encode('utf-8', 'replace').replace('?', '_')
 
 
-def xml_name(p, keep_escape=False):
+def xml_name(p, keep_escape=False, encoding=None):
     """ Prepare name for use in HTML/XML contect """
 
-    if type(p) != type(u''):
-        if sabnzbd.DARWIN:
+    if isinstance(p, unicode):
+        pass
+    elif isinstance(p, str):
+        if sabnzbd.DARWIN or encoding == 'utf-8':
+            p = p.decode('utf-8', 'replace')
+        elif gUTF:
             p = p.decode('utf-8', 'replace')
         else:
             p = p.decode('Latin-1', 'replace')
+    else:
+        p = str(p)
 
     if keep_escape:
         return p.encode('ascii', 'xmlcharrefreplace')
@@ -78,18 +112,41 @@ def xml_name(p, keep_escape=False):
         return escape(p).encode('ascii', 'xmlcharrefreplace')
 
 
+def latin1(txt):
+    """ When Unicode, convert to Latin-1 """
+    if isinstance(txt, unicode):
+        return txt.encode('latin-1', 'replace')
+    else:
+        return txt
+
 
 def encode_for_xml(ustr, encoding='ascii'):
     """
     Encode unicode_data for use as XML or HTML, with characters outside
     of the encoding converted to XML numeric character references.
     """
-    if type(ustr) != type(u''):
+    if isinstance(ustr, unicode):
+        pass
+    elif isinstance(ustr, str):
         ustr = ustr.decode('Latin-1', 'replace')
-
+    else:
+        ustr = unicode(str(ustr))
     return ustr.encode(encoding, 'xmlcharrefreplace')
 
 
+def titler(p):
+    """ title() replacement
+        Python's title() fails with Latin-1, so use Unicode detour.
+    """
+    if gUTF:
+        try:
+            return p.decode('utf-8').title().encode('utf-8')
+        except:
+            return p.decode('latin-1', 'replace').title().encode('latin-1', 'replace')
+    else:
+        return p.decode('latin-1', 'replace').title().encode('latin-1', 'replace')
+
+
 class LatinFilter(Filter):
     """ Make sure Cheetah gets only Unicode strings """
     def filter(self, val,
@@ -135,8 +192,8 @@ gTABLE850 = string.maketrans(
 def TRANS(p):
     """ For Windows: Translate CP850 to Python's Latin-1
     """
-    global gTABLE850, gNT
-    if gNT:
+    global gTABLE850
+    if sabnzbd.WIN32:
         return p.translate(gTABLE850)
     else:
         return p
