@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -16,7 +16,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 CONFIG_VERSION = 18
-QUEUE_VERSION = 7
+QUEUE_VERSION = 9
+POSTPROC_QUEUE_VERSION = 1
 
 PNFO_REPAIR_FIELD = 0
 PNFO_UNPACK_FIELD = 1
@@ -35,6 +36,7 @@ PNFO_FINISHED_FILES_FIELD = 13
 PNFO_ACTIVE_FILES_FIELD = 14
 PNFO_QUEUED_FILES_FIELD = 15
 PNFO_STATUS_FIELD = 16
+PNFO_PRIORITY_FIELD = 17
 
 QNFO_BYTES_FIELD = 0
 QNFO_BYTES_LEFT_FIELD = 1
@@ -48,32 +50,43 @@ GIGI = float(2 ** 30)
 MEBI = float(2 ** 20)
 KIBI = float(2 ** 10)
 
-STAGENAMES = {0:"Download", 1:"Par2", 2:"Unrar", 3:"Unzip", 4:"Filejoin", 5:"UserScript"}
-
 BYTES_FILE_NAME  = 'bytes%s.sab' % QUEUE_VERSION
-QUEUE_FILE_NAME  = 'queue%s.sab' % QUEUE_VERSION
+QUEUE_FILE_TMPL  = 'queue%s.sab'
+QUEUE_FILE_NAME  =  QUEUE_FILE_TMPL % QUEUE_VERSION
+POSTPROC_QUEUE_FILE_NAME  = 'postproc%s.sab' % POSTPROC_QUEUE_VERSION
 RSS_FILE_NAME    = 'rss_data.sab'
 BOOKMARK_FILE_NAME = 'bookmarks.sab'
 SCAN_FILE_NAME    = 'watched_data.sab'
 
+DB_HISTORY_VERSION = 1
+DB_QUEUE_VERSION = 1
+
+DB_HISTORY_NAME = 'history%s.db' % DB_HISTORY_VERSION
+DB_QUEUE_NAME = 'queue%s.db' % DB_QUEUE_VERSION
+
 DEF_DOWNLOAD_DIR = 'downloads/incomplete'
 DEF_COMPLETE_DIR = 'downloads/complete'
 DEF_CACHE_DIR    = 'cache'
 DEF_LOG_DIR      = 'logs'
 DEF_NZBBACK_DIR  = ''
+DEF_LANGUAGE     = 'language'
 DEF_INTERFACES   = 'interfaces'
-DEF_STDINTF      = 'Default'
+DEF_INT_LANGUAGE = 'language'
+DEF_STDINTF      = 'smpl'
+DEF_SKIN_COLORS  = {'smpl':'white', 'classic':'darkblue', 'mobile':'light'}
 DEF_MAIN_TMPL    = 'templates/main.tmpl'
 DEF_INI_FILE     = 'sabnzbd.ini'
 DEF_HOST         = 'localhost'
 DEF_PORT_WIN     = 8080
 DEF_PORT_UNIX    = 8080
+DEF_PORT_WIN_SSL = 9090
+DEF_PORT_UNIX_SSL= 9090
 DEF_WORKDIR      = 'sabnzbd'
 DEF_LOG_FILE     = 'sabnzbd.log'
 DEF_LOG_ERRFILE  = 'sabnzbd.error.log'
 DEF_LOG_CHERRY   = 'cherrypy.log'
-DEF_TIMEOUT      = 120
-MIN_TIMEOUT      = 30
+DEF_TIMEOUT      = 60
+MIN_TIMEOUT      = 10
 MAX_TIMEOUT      = 200
 DEF_LOGLEVEL     = 1
 DEF_SCANRATE     = 5
@@ -82,7 +95,23 @@ MIN_DECODE_QUEUE = 5
 MAX_DECODE_QUEUE = 10
 MAX_WARNINGS     = 20
 
+TOP_PRIORITY = 2
+HIGH_PRIORITY = 1
+NORMAL_PRIORITY = 0
+LOW_PRIORITY = -1
+DEFAULT_PRIORITY = -100
+
+series_match = [ ('([sS]|[\d]+)x(\d+)', # 1x01
+                     ['^[-\.]+([sS]|[\d])+x(\d+)', '^[-\.](\d+)'] ),  #(MATCHER, [EXTRA,MATCHERS])
+
+                      ('[Ss](\d+)[\.\-]?[Ee](\d+)',  # S01E01
+                       ['^[-\.]+[Ss](\d+)[\.\-]?[Ee](\d+)', '^[-\.](\d+)']) ] # Extra matchers
 
-tv_episode_match = ['([sS]|[\d]+)x(\d+)',# 1x01
-                      '[Ss](\d+)[\.\-]?[Ee](\d+)'] # S01E01
                       #possibly flawed - 101 - support: [\.\- \s]?(\d)(\d{2,2})[\.\- \s]?
+
+date_match = ['(\d{4})\W(\d{1,2})\W(\d{1,2})', #2008-10-16
+              '(\d{1,2})\W(\d{1,2})\W(\d{4})'] #10.16.2008
+
+year_match = '[\W]([1|2]\d{3})([^\w]|$)' # Something '(YYYY)' or '.YYYY.' or ' YYYY '
+
+sample_match = '((^|[\W_])sample\d*[\W_])|(-s\.)' # something-sample.avi something-s.avi
