@@ -19,18 +19,11 @@
 sabnzbd.decoder - article decoder
 """
 
-__NAME__ = 'decoder'
-
 import Queue
 import binascii
 import logging
 import re
-import sabnzbd
-from sabnzbd.constants import *
-from sabnzbd.codecs import name_fixer
-
 from threading import Thread
-
 try:
     import _yenc
     HAVE_YENC = True
@@ -38,6 +31,15 @@ try:
 except ImportError:
     HAVE_YENC = False
 
+import sabnzbd
+from sabnzbd.constants import *
+import sabnzbd.articlecache as articlecache
+import sabnzbd.downloader
+import sabnzbd.cfg as cfg
+import sabnzbd.nzbqueue
+from sabnzbd.codecs import name_fixer
+from sabnzbd.lang import Ta
+
 #-------------------------------------------------------------------------------
 
 class CrcError(Exception):
@@ -57,13 +59,13 @@ class Decoder(Thread):
     def __init__(self, servers):
         Thread.__init__(self)
 
-        self.queue = Queue.Queue();
+        self.queue = Queue.Queue()
         self.servers = servers
 
     def decode(self, article, lines):
         self.queue.put((article, lines))
         if self.queue.qsize() > MAX_DECODE_QUEUE:
-            sabnzbd.delay_downloader()
+            sabnzbd.downloader.delay_downloader()
 
     def stop(self):
         self.queue.put(None)
@@ -74,8 +76,8 @@ class Decoder(Thread):
             if not art_tup:
                 break
 
-            if self.queue.qsize() < MIN_DECODE_QUEUE and sabnzbd.delayed():
-                sabnzbd.undelay_downloader()
+            if self.queue.qsize() < MIN_DECODE_QUEUE and sabnzbd.downloader.delayed():
+                sabnzbd.downloader.undelay_downloader()
 
             article, lines = art_tup
             nzf = article.nzf
@@ -87,44 +89,40 @@ class Decoder(Thread):
 
             if lines:
                 try:
-                    logging.info("[%s] Decoding %s", __NAME__, article)
+                    logging.info("Decoding %s", article)
 
                     data = decode(article, lines)
                     nzf.increase_article_count()
                 except IOError, e:
-                    logging.error("[%s] Decoding %s failed", __NAME__,
-                                      article)
-                    sabnzbd.pause_downloader()
+                    logging.error(Ta('error-decode@1'), article)
+                    sabnzbd.downloader.pause_downloader()
 
                     article.fetcher = None
 
-                    sabnzbd.reset_try_lists(nzf, nzo)
+                    sabnzbd.nzbqueue.reset_try_lists(nzf, nzo)
 
                     register = False
 
                 except CrcError, e:
-                    logging.warning("[%s] CRC Error in %s (%s -> %s)", __NAME__,
-                                    article, e.needcrc, e.gotcrc)
+                    logging.warning(Ta('warn-crcError@3'), article, e.needcrc, e.gotcrc)
 
                     data = e.data
 
-                    if sabnzbd.FAIL_ON_CRC:
+                    if cfg.FAIL_ON_CRC.get():
                         new_server_found = self.__search_new_server(article)
                         if new_server_found:
                             register = False
 
                 except BadYenc, e:
-                    logging.warning("[%s] Badly formed yEnc article in %s", __NAME__,
-                                    article)
+                    logging.warning("Badly formed yEnc article in %s", article)
 
-                    if sabnzbd.FAIL_ON_CRC:
+                    if cfg.FAIL_ON_CRC.get():
                         new_server_found = self.__search_new_server(article)
                         if new_server_found:
                             register = False
 
                 except:
-                    logging.error("[%s] Unknown Error while decoding %s",
-                                      __NAME__, article)
+                    logging.error(Ta('error-decodeUnknown@1'), article)
 
             else:
                 new_server_found = self.__search_new_server(article)
@@ -132,10 +130,10 @@ class Decoder(Thread):
                     register = False
 
             if data:
-                sabnzbd.save_article(article, data)
+                articlecache.method.save_article(article, data)
 
             if register:
-                sabnzbd.register_article(article)
+                sabnzbd.nzbqueue.register_article(article)
 
     def __search_new_server(self, article):
         article.add_to_try_list(article.fetcher)
@@ -147,7 +145,7 @@ class Decoder(Thread):
         fill_server_found = False
 
         for server in self.servers:
-            if not article.server_in_try_list(server):
+            if server.active and not article.server_in_try_list(server):
                 if server.fillserver:
                     fill_server_found = True
                 else:
@@ -163,19 +161,17 @@ class Decoder(Thread):
             article.fetcher = None
 
             ## Allow all servers to iterate over this nzo and nzf again ##
-            sabnzbd.reset_try_lists(nzf, nzo)
+            sabnzbd.nzbqueue.reset_try_lists(nzf, nzo)
 
-            logging.info('[%s] %s => found at least one untested server',
-                            __NAME__, article)
+            logging.info('%s => found at least one untested server', article)
 
         else:
-            logging.warning('[%s] %s => missing from all servers, discarding',
-                            __NAME__, article)
+            logging.warning(Ta('warn-artAllMissing@1'), article)
 
         return new_server_found
 #-------------------------------------------------------------------------------
 
-YDEC_TRANS = ''.join([chr((i + 256 - 42) % 256) for i in range(256)])
+YDEC_TRANS = ''.join([chr((i + 256 - 42) % 256) for i in xrange(256)])
 def decode(article, data):
     data = strip(data)
     ## No point in continuing if we don't have any data left
@@ -188,10 +184,16 @@ def decode(article, data):
 
         #Deal with non-yencoded posts
         if not ybegin:
-            if data[0].startswith('begin '):
-                nzf.set_filename(name_fixer(data[0].split(None, 2)[2]))
-                nzf.set_type('uu')
-                data.pop(0)
+            found = False
+            for i in xrange(10):
+                if data[i].startswith('begin '):
+                    nzf.set_filename(name_fixer(data[i].split(None, 2)[2]))
+                    nzf.set_type('uu')
+                    found = True
+                    break
+            if found:
+                for n in xrange(i):
+                    data.pop(0)
             if data[-1] == 'end':
                 data.pop()
                 if data[-1] == '`':
@@ -204,8 +206,8 @@ def decode(article, data):
             if 'name' in ybegin:
                 nzf.set_filename(name_fixer(ybegin['name']))
             else:
-                logging.debug("[%s] Possible corrupt header detected " + \
-                              "=> ybegin: %s", __NAME__, ybegin)
+                logging.debug("Possible corrupt header detected " + \
+                              "=> ybegin: %s", ybegin)
             nzf.set_type('yenc')
             # Decode data
             if HAVE_YENC:
@@ -216,9 +218,9 @@ def decode(article, data):
                 for i in (0, 9, 10, 13, 27, 32, 46, 61):
                     j = '=%c' % (i + 64)
                     data = data.replace(j, chr(i))
-                    decoded_data = data.translate(YDEC_TRANS)
-                    crc = binascii.crc32(decoded_data)
-                    partcrc = '%08X' % (crc & 2**32L - 1)
+                decoded_data = data.translate(YDEC_TRANS)
+                crc = binascii.crc32(decoded_data)
+                partcrc = '%08X' % (crc & 2**32L - 1)
 
             if ypart:
                 crcname = 'pcrc32'
@@ -229,8 +231,8 @@ def decode(article, data):
                 _partcrc = '0' * (8 - len(yend[crcname])) + yend[crcname].upper()
             else:
                 _partcrc = None
-                logging.debug("[%s] Corrupt header detected " + \
-                              "=> yend: %s", __NAME__, yend)
+                logging.debug("Corrupt header detected " + \
+                              "=> yend: %s", yend)
 
             if not (_partcrc == partcrc):
                 raise CrcError(_partcrc, partcrc, decoded_data)
