@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2009 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,35 +22,75 @@ sabnzbd.newswrapper
 import errno
 import socket
 from threading import Thread
-
 from nntplib import NNTPPermanentError
-from time import time
-from sabnzbd.constants import *
-import sabnzbd
+import time
 import logging
 
+import sabnzbd
+from sabnzbd.constants import *
+from sabnzbd.lang import Ta
+
 try:
     from OpenSSL import SSL
     _ssl = SSL
+    WantReadError = _ssl.WantReadError
     del SSL
     HAVE_SSL = True
 
 except ImportError:
     _ssl = None
     HAVE_SSL = False
+    
+    # Dummy class so this exception is ignored by clients without ssl installed
+    class WantReadError(Exception):
+        def __init__(self, value):
+            self.parameter = value
+        def __str__(self):
+            return repr(self.parameter)
 
 import threading
 _RLock = threading.RLock
 del threading
 
 import select
-import os
 
-__NAME__ = "newswrapper"
 
 socket.setdefaulttimeout(DEF_TIMEOUT)
 
+
+#------------------------------------------------------------------------------
+# getaddrinfo() can be very slow. In some situations this can lead
+# to delayed starts and timeouts on connections.
+# Because of this, the results will be cached in the server object.
+
+def _retrieve_info(server):
+    """ Async attempt to run getaddrinfo() for specified server
+    """
+    info = GetServerParms(server.host, server.port)
+
+    if info is None:
+        server.bad_cons += server.threads
+    else:
+        server.bad_cons = 0
+    (server.info, server.request) = (info, False)
+    sabnzbd.downloader.wakeup()
+
+
+def request_server_info(server):
+    """ Launch async request to resolve server address
+    """
+    if not server.request:
+        server.request = True
+        Thread(target=_retrieve_info, args=(server,)).start()
+
+
 def GetServerParms(host, port):
+    # Make sure port is numeric (unicode input not supported)
+    try:
+        int(port)
+    except:
+        # Could do with a warning here
+        port = 119
     try:
         # Standard IPV4
         return socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
@@ -73,70 +113,113 @@ def con(sock, host, port, sslenabled, nntp):
                 try:
                     sock.do_handshake()
                     break
-                except _ssl.WantReadError:
+                except WantReadError:
                     select.select([sock], [], [], 1.0)
     except socket.error, e:
         try:
-            (_errno, strerror) = e
+            # socket.error can either return a string or a tuple
+            if isinstance(e, tuple):
+                (_errno, strerror) = e
+            else:
+                # Are we safe to hardcode the ETIMEDOUT error?
+                (_errno, strerror) = (errno.ETIMEDOUT, str(e))
+                e = (_errno, strerror)
             #expected, do nothing
             if _errno == errno.EINPROGRESS:
                 pass
         finally:
             nntp.error(e)
 
+    except _ssl.Error, e:
+        nntp.error(e)
+
 class NNTP:
-    def __init__(self, host, port, sslenabled, nntp, user=None, password=None):
+    def __init__(self, host, port, info, sslenabled, nntp, user=None, password=None, block=False):
         self.host = host
         self.port = port
         self.nntp = nntp
-        res= GetServerParms(self.host, self.port)
-        if not res:
-            raise socket.error(errno.EADDRNOTAVAIL, "Address not available - Check for internet or DNS problems")
+        self.blocking = block
+        self.error_msg = None
+        if not info:
+            if block:
+                info = GetServerParms(host, port)
+            else:
+                raise socket.error(errno.EADDRNOTAVAIL, "Address not available - Check for internet or DNS problems")
 
-        af, socktype, proto, canonname, sa = res[0]
+        af, socktype, proto, canonname, sa = info[0]
 
         if sslenabled and _ssl:
-            ctx = _ssl.Context(_ssl.SSLv23_METHOD)
+            # Some users benefit from SSLv2 not being capped.
+            ssl_type = sabnzbd.cfg.SSL_TYPE.get()
+            if ssl_type == 'v2':
+                ctx = _ssl.Context(_ssl.SSLv2_METHOD)
+            elif ssl_type == 'v3':
+                ctx = _ssl.Context(_ssl.SSLv3_METHOD)
+            else:
+                ctx = _ssl.Context(_ssl.SSLv23_METHOD)
+
             self.sock = SSLConnection(ctx, socket.socket(af, socktype, proto))
         elif sslenabled and not _ssl:
-            logging.error("[%s] Error importing OpenSSL module. Connecting with NON-SSL", __NAME__)
+            logging.error(Ta('error-noSSL'))
             self.sock = socket.socket(af, socktype, proto)
         else:
             self.sock = socket.socket(af, socktype, proto)
 
         try:
-            if os.name == 'nt':
+            # Windows must do the connection in a seperate thread due to non-blocking issues
+            # If the server wants to be blocked (for testing) then use the linux route
+            if sabnzbd.WIN32 and not block:
                 Thread(target=con, args=(self.sock, self.host, self.port, sslenabled, self)).start()
-            else: 
+            else:
+                # if blocking (server test) only wait for 4 seconds during connect until timeout
+                if block:
+                    self.sock.settimeout(10)
                 self.sock.connect((self.host, self.port))
-                self.sock.setblocking(0)
+                if not block:
+                    self.sock.setblocking(0)
                 if sslenabled and _ssl:
                     while True:
                         try:
                             self.sock.do_handshake()
                             break
-                        except _ssl.WantReadError:
+                        except WantReadError:
                             select.select([self.sock], [], [], 1.0)
 
         except socket.error, e:
             try:
-                (_errno, strerror) = e
+                # socket.error can either return a string or a tuple
+                if isinstance(e, tuple):
+                    (_errno, strerror) = e
+                else:
+                    # Are we safe to hardcode the ETIMEDOUT error?
+                    (_errno, strerror) = (errno.ETIMEDOUT, str(e))
+                    e = (_errno, strerror)
                 #expected, do nothing
                 if _errno == errno.EINPROGRESS:
                     pass
             finally:
                 self.error(e)
-                
+
+        except _ssl.Error, e:
+            self.error(e)
+
     def error(self, error):
-        msg = "Failed to connect: %s" % (error)
-        logging.error("[%s] %s %s@%s:%s",
-                          __NAME__, msg, self.nntp.thrdnum, self.host,
-                          self.port)
+        if 'SSL23_GET_SERVER_HELLO' in str(error):
+            error = 'This server does not allow SSL on this port'
+        msg = "Failed to connect: %s" % (str(error))
+        msg = "%s %s@%s:%s" % (msg, self.nntp.thrdnum, self.host, self.port)
+        self.error_msg = msg
+        if self.blocking:
+            raise socket.error(errno.ECONNREFUSED, msg)
+        else:
+            logging.info(msg)
+            self.nntp.server.warning = msg
 
 class NewsWrapper:
-    def __init__(self, server, thrdnum):
+    def __init__(self, server, thrdnum, block=False):
         self.server = server
         self.thrdnum = thrdnum
+        self.blocking = block
 
         self.timeout = None
         self.article = None
@@ -157,11 +240,11 @@ class NewsWrapper:
         self.pass_ok = False
 
     def init_connect(self):
-        self.nntp = NNTP(self.server.host, self.server.port, self.server.ssl, self,
-                         self.server.username, self.server.password)
+        self.nntp = NNTP(self.server.host, self.server.port, self.server.info, self.server.ssl, self,
+                         self.server.username, self.server.password, self.blocking)
         self.recv = self.nntp.sock.recv
 
-        self.timeout = time() + self.server.timeout
+        self.timeout = time.time() + self.server.timeout
 
     def finish_connect(self):
         if not self.server.username or not self.server.password:
@@ -171,7 +254,9 @@ class NewsWrapper:
             self.pass_sent = True
             self.pass_ok = True
 
-        if not self.user_sent:
+        if self.lines and self.lines[0][:3] == '400':
+            raise NNTPPermanentError(self.lines[0])
+        elif not self.user_sent:
             command = 'authinfo user %s\r\n' % (self.server.username)
             self.nntp.sock.sendall(command)
             self.user_sent = True
@@ -185,30 +270,38 @@ class NewsWrapper:
             self.pass_sent = True
         elif self.user_ok and not self.pass_ok:
             if self.lines[0][:3] != '281':
+                # Assume that login failed (code 481 or other)
                 raise NNTPPermanentError(self.lines[0])
             else:
                 self.connected = True
 
-        self.timeout = time() + self.server.timeout
+        self.timeout = time.time() + self.server.timeout
 
     def body(self):
-        self.timeout = time() + self.server.timeout
+        self.timeout = time.time() + self.server.timeout
         command = 'BODY <%s>\r\n' % (self.article.article)
         self.nntp.sock.sendall(command)
 
     def send_group(self, group):
-        self.timeout = time() + self.server.timeout
+        self.timeout = time.time() + self.server.timeout
         command = 'GROUP %s\r\n' % (group)
         self.nntp.sock.sendall(command)
 
-    def recv_chunk(self):
-        self.timeout = time() + self.server.timeout
+    def recv_chunk(self, block=False):
+        self.timeout = time.time() + self.server.timeout
         while 1:
             try:
                 chunk = self.recv(32768)
                 break
-            except _ssl.WantReadError:
-                return (0, False, True)
+            except WantReadError:
+                # SSL connections will block until they are ready.
+                # Either ignore the connection until it responds
+                # Or wait in a loop until it responds
+                if block:
+                    #time.sleep(0.0001)
+                    continue
+                else:
+                    return (0, False, True)
 
         self.data += chunk
         new_lines = self.data.split('\r\n')
@@ -228,11 +321,12 @@ class NewsWrapper:
         self.data = ''
         self.lines = []
 
-    def hard_reset(self, wait=True):
+    def hard_reset(self, wait=True, quit=True):
         if self.nntp:
             try:
-                self.nntp.sock.sendall('QUIT\r\n')
-                time.sleep(0.1)
+                if quit:
+                    self.nntp.sock.sendall('QUIT\r\n')
+                    time.sleep(0.1)
                 self.nntp.sock.close()
             except:
                 pass
@@ -242,10 +336,23 @@ class NewsWrapper:
         # Wait before re-using this newswrapper
         if wait:
             # Reset due to error condition, use server timeout
-            self.timeout = time() + self.server.timeout
+            self.timeout = time.time() + self.server.timeout
         else:
             # Reset for internal reasons, just wait 5 sec
-            self.timeout = time() + 5
+            self.timeout = time.time() + 5
+
+    def terminate(self, quit=False):
+        """ Close connection and remove nntp object """
+        if self.nntp:
+            try:
+                if quit:
+                    self.nntp.sock.sendall('QUIT\r\n')
+                    time.sleep(0.1)
+                self.nntp.sock.close()
+            except:
+                pass
+        del self.nntp
+
 
 class SSLConnection:
     def __init__(self, *args):
@@ -259,10 +366,10 @@ class SSLConnection:
               'makefile', 'get_app_data', 'set_app_data', 'state_string',
               'sock_shutdown', 'get_peer_certificate', 'want_read',
               'want_write', 'set_connect_state', 'set_accept_state',
-              'connect_ex', 'sendall', 'do_handshake'):
+              'connect_ex', 'sendall', 'do_handshake', 'settimeout'):
         exec """def %s(self, *args):
             self._lock.acquire()
             try:
                 return apply(self._ssl_conn.%s, args)
             finally:
-                self._lock.release()\n""" % (f, f)
\ No newline at end of file
+                self._lock.release()\n""" % (f, f)
