@@ -18,30 +18,37 @@
 """
 sabnzbd.nzbstuff - misc
 """
+
+import os
 import time
 import re
 import logging
 import sabnzbd
 import datetime
-from sabnzbd.constants import *
+import xml.sax
+import xml.sax.handler
+import xml.sax.xmlreader
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
+import sabnzbd
+from sabnzbd.constants import *
+import sabnzbd.misc
+import sabnzbd.config as config
+import sabnzbd.cfg as cfg
 from sabnzbd.trylist import TryList
-from sabnzbd.utils import listquote
+from sabnzbd.lang import T, Ta
+from sabnzbd.misc import to_units, cat_to_opts
+from sabnzbd.codecs import unicoder
 
 RE_NEWZBIN = re.compile(r"msgid_(\w+) (.+)(\.nzb)$", re.I)
 RE_NORMAL  = re.compile(r"(.+)(\.nzb)", re.I)
+SUBJECT_FN_MATCHER = re.compile(r'"(.*)"')
+RE_SAMPLE = re.compile(sample_match, re.I)
+PROBABLY_PAR2_RE = re.compile(r'(.*)\.vol(\d*)\+(\d*)\.par2', re.I)
 
-HAVE_CELEMENTTREE = True
-try:
-    from xml.etree.cElementTree import XML
-except ImportError:
-    try:
-        from cElementTree import XML
-    except ImportError:
-        from elementtree.ElementTree import XML
-        HAVE_CELEMENTTREE = False
-
-__NAME__ = "nzbstuff"
 
 ################################################################################
 # Article                                                                      #
@@ -60,7 +67,7 @@ class Article(TryList):
         self.nzf = nzf
 
     def get_article(self, server):
-        if server.fillserver and not self.allow_fill_server:
+        if server.fillserver and (not self.allow_fill_server) and sabnzbd.active_primaries():
             return None
 
         if not self.fetcher and not self.server_in_try_list(server):
@@ -93,7 +100,7 @@ class Article(TryList):
 ################################################################################
 SUBJECT_FN_MATCHER = re.compile(r'"(.*)"')
 class NzbFile(TryList):
-    def __init__(self, date, subject, segments, nzo):
+    def __init__(self, date, subject, article_db, bytes, nzo):
         TryList.__init__(self)
 
         # Private
@@ -104,7 +111,7 @@ class NzbFile(TryList):
 
         match = re.search(SUBJECT_FN_MATCHER, subject)
         if match:
-            self.__filename = match.group(1)
+            self.__filename = match.group(1).strip('"')
 
         self.__ispar2file = False
         self.__vol = None
@@ -117,8 +124,8 @@ class NzbFile(TryList):
         self.__articles = []
         self.__decodetable = {}
 
-        self.__bytes = 0
-        self.__bytes_left = 0
+        self.__bytes = bytes
+        self.__bytes_left = bytes
         self.__article_count = 0
 
         # Public
@@ -129,27 +136,9 @@ class NzbFile(TryList):
         self.valid = False
         self.import_finished = False
 
-        if segments:
-            # Do a lazy import
-            article_db = {}
-            for segment in segments:
-                bytes = int(segment.get('bytes'))
-                partnum = int(segment.get('number'))
+        self.md5sum = None
 
-                if partnum in article_db:
-                    if segment.text == article_db[partnum][0]:
-                        logging.warning("[%s] Skipping duplicate article (%s)", __NAME__, segment.text)
-                    else:
-                        logging.error("[%s] INCORRECT NZB: Duplicate part %s, but different ID-s (%s // %s)",
-                                         __NAME__, partnum, article_db[partnum][0], segment.text)
-                    continue
-
-                self.__bytes += bytes
-                self.__bytes_left += bytes
-
-                self.valid = True
-
-                article_db[partnum] = (segment.text, bytes)
+        self.valid = bool(article_db)
 
         if self.valid and self.nzf_id:
             sabnzbd.save_data(article_db, self.nzf_id)
@@ -160,7 +149,7 @@ class NzbFile(TryList):
         self.__article_count += 1
 
     def finish_import(self):
-        logging.info("[%s] Finishing import on %s", __NAME__, self.__subject)
+        logging.info("Finishing import on %s", self.__subject)
 
         article_db = sabnzbd.load_data(self.nzf_id)
         if article_db:
@@ -234,6 +223,11 @@ class NzbFile(TryList):
 
         self.add_to_try_list(server)
 
+    def reset_all_try_lists(self):
+        for art in self.__articles:
+            art.reset_try_list()
+        self.reset_try_list()
+
     def bytes(self):
         return self.__bytes
 
@@ -283,40 +277,184 @@ class NzbFile(TryList):
         return "<NzbFile: filename=%s, type=%s>" % \
                (self.__filename, self.__type)
 
+
+################################################################################
+# NzbParser                                                                    #
+################################################################################
+class NzbParser(xml.sax.handler.ContentHandler):
+    """ Forgiving parser for NZB's """
+    # Accesses private variables of NzbObject instances to keep
+    # queue-compatibility with previous trunk versions.
+    # Ideally the methods of this class could be added to NzbObject,
+    # but this would also break compatibility.
+    # Hence, this solution.
+    def __init__ (self, nzo):
+        self.nzo = nzo
+        self.in_nzb = False
+        self.in_file = False
+        self.in_groups = False
+        self.in_group = False
+        self.in_segments = False
+        self.in_segment = False
+        self.filename = ''
+        self.avg_age = 0
+        self.valids = 0
+        self.skipped_files = 0
+        self.nzf_list = []
+
+    def startDocument(self):
+        self.filter = cfg.IGNORE_SAMPLES.get()
+
+    def startElement(self, name, attrs):
+        if name == 'segment' and self.in_nzb and self.in_file and self.in_segments:
+            try:
+                self.seg_bytes = int(attrs.get('bytes'))
+                self.article_nr = int(attrs.get('number'))
+            except ValueError:
+                return
+            self.article_id = []
+            self.file_bytes += self.seg_bytes
+            self.in_segment = True
+
+        elif name == 'segments' and self.in_nzb and self.in_file:
+            self.in_segments = True
+
+        elif name == 'file' and self.in_nzb:
+            subject = attrs.get('subject', '')
+            match = re.search(SUBJECT_FN_MATCHER, subject)
+            if match:
+                self.filename = match.group(1).strip('"').strip()
+            else:
+                self.filename = subject.strip()
+
+            if self.filter == 2 and RE_SAMPLE.search(subject):
+                logging.info('Skipping sample file %s', subject)
+            else:
+                self.in_file = True
+                if isinstance(subject, unicode):
+                    subject = subject.encode('latin-1', 'replace')
+                self.fileSubject = subject
+                try:
+                    self.file_date = int(attrs.get('date'))
+                except:
+                    # NZB has non-standard timestamp, assume 1
+                    self.file_date = 1
+                self.article_db = {}
+                self.file_bytes = 0
+
+        elif name == 'group' and self.in_nzb and self.in_file and self.in_groups:
+            self.in_group = True
+            self.group_name = []
+
+        elif name == 'groups' and self.in_nzb and self.in_file:
+            self.in_groups = True
+            self.groups = []
+
+        elif name == 'nzb':
+            self.in_nzb = True
+
+    def characters (self, content):
+        if self.in_group:
+            self.group_name.append(content)
+        elif self.in_segment:
+            self.article_id.append(content)
+
+    def endElement(self, name):
+        if name == 'group' and self.in_group:
+            self.groups.append(str(''.join(self.group_name)))
+            self.in_group = False
+
+        elif name == 'segment' and self.in_segment:
+            partnum = self.article_nr
+            segm = str(''.join(self.article_id))
+            if partnum in self.article_db:
+                if segm != self.article_db[partnum][0]:
+                    logging.error(Ta('error-qDupPart@3'),
+                                         partnum, self.article_db[partnum][0], segm)
+                else:
+                    logging.info("Skipping duplicate article (%s)", segm)
+            else:
+                self.article_db[partnum] = (segm, self.seg_bytes)
+            self.in_segment = False
+
+        elif name == 'groups' and self.in_groups:
+            self.in_groups = False
+
+        elif name == 'segments' and self.in_segments:
+            self.in_segments = False
+
+        elif name == 'file' and self.in_file:
+            # Create an NZF
+            self.in_file = False
+            if not self.article_db:
+                logging.warning(Ta('warn-emptyFile@1'), self.filename)
+                return
+            tm = datetime.datetime.fromtimestamp(self.file_date)
+            nzf = NzbFile(tm, self.filename, self.article_db, self.file_bytes, self.nzo)
+            if nzf.valid and nzf.nzf_id:
+                logging.info('File %s added to queue', self.filename)
+                self.nzo._NzbObject__files.append(nzf)
+                self.nzo._NzbObject__files_table[nzf.nzf_id] = nzf
+                self.nzo._NzbObject__bytes += nzf.bytes()
+                self.avg_age += self.file_date
+                self.valids += 1
+                self.nzf_list.append(nzf)
+            else:
+                logging.info('Error importing %s, skipping', self.filename)
+                if nzf.nzf_id:
+                    sabnzbd.remove_data(nzf.nzf_id)
+                self.skipped_files += 1
+
+        elif name == 'nzb':
+            self.in_nzb = False
+
+    def endDocument(self):
+        """ End of the file """
+        self.nzo._NzbObject__group = self.groups
+        self.nzo._NzbObject__avg_date = datetime.datetime.fromtimestamp(self.avg_age / self.valids)
+        if self.skipped_files:
+            logging.warning(Ta('warn-badImport@2'),
+                            self.skipped_files, self.nzo.get_filename())
+
+    def remove_files(self):
+        """ Remove all created NZF objects """
+        for nzf in self.nzf_list:
+            sabnzbd.remove_data(nzf.nzf_id)
+
+
 ################################################################################
 # NzbObject                                                                    #
 ################################################################################
-DTD = '{http://www.newzbin.com/DTD/2003/nzb}'
-NZBFN_MATCHER = re.compile(r"msgid_\d*_(.*).nzb", re.I)
-PROBABLY_PAR2_RE = re.compile(r'(.*)\.vol(\d*)\+(\d*)\.par2', re.I)
 
 class NzbObject(TryList):
-    def __init__(self, filename, pp, script, nzb = None,
-                 futuretype = False, cat = None, url=None):
+    def __init__(self, filename, msgid, pp, script, nzb = None,
+                 futuretype = False, cat = None, url=None,
+                 priority=NORMAL_PRIORITY, nzbname=None, status="Queued", nzo_info=None):
         TryList.__init__(self)
 
-        if cat and pp == None:
-            try:
-                pp = sabnzbd.CFG['categories'][cat.lower()]['pp']
-            except:
-                pass
-        if cat and script == None:
-            try:
-                script = sabnzbd.CFG['categories'][cat.lower()]['script']
-            except:
-                pass
-
-        if pp == None and futuretype:
+        if pp is None:
             r = u = d = None
         else:
             r, u, d = sabnzbd.pp_to_opts(pp)
 
-        if script == None and not futuretype:
-            script = sabnzbd.DIRSCAN_SCRIPT
-
         self.__filename = filename    # Original filename
-        self.__dirname = filename
-        self.__original_dirname = filename
+        if nzbname and nzb:
+            dirname = nzbname         # Use nzbname if set and only for non-future slot
+        else:
+            dirname = filename
+
+        # If non-future: create safe folder name stripped from ".nzb" and junk
+        if nzb and dirname and dirname.lower().endswith('.nzb'):
+            dname, ext = os.path.splitext(dirname) # Used for folder name for final unpack
+            if ext.lower() == '.nzb':
+                dirname = dname
+            dirname = sabnzbd.misc.sanitize_foldername(dirname)
+        dirname, password = scan_password(dirname)
+
+        self.__dirname = dirname      # Keeps track of the working folder
+        self.__original_dirname = dirname # TAKE NOTE: Used for folder name for final unpack
+                                          # The name is wrong, required for backward compatibility!
+
         self.__created = False        # dirprefixes + dirname created
         self.__bytes = 0              # Original bytesize
         self.__bytes_downloaded = 0   # Downloaded byte
@@ -330,25 +468,28 @@ class NzbObject(TryList):
             self.__url = str(url)     # Either newzbin-id or URL queued (future-type only)
         else:
             self.__url = ''
-        self.__group = None
-        self.__avg_date = None
+        self.__group = []
+        self.__avg_date = datetime.datetime.fromtimestamp(0.0)
         self.__dirprefix = []
 
-        self.__partable = {}
-        self.__extrapars = {}
+        self.__partable = {}          # Holds one parfile-name for each set
+        self.__extrapars = {}         # Holds the extra parfile names for all sets
+        self.md5packs = {}            # Holds the md5pack for each set
 
         self.__files = []
         self.__files_table = {}
 
         self.__finished_files = []
 
-        self.__unpackstrht = {}
-
         #the current status of the nzo eg:
         #Queued, Downloading, Repairing, Unpacking, Failed, Complete
-        self.__status = "Queued"
+        self.__status = status
         self.__avg_bps_freq = 0
         self.__avg_bps_total = 0
+        try:
+            self.__priority = int(priority)
+        except:
+            self.__priority = DEFAULT_PRIORITY
 
         self.__dupe_table = {}
 
@@ -360,120 +501,98 @@ class NzbObject(TryList):
         self.deleted = False
         self.parsed = False
 
-        self.create_group_folder = sabnzbd.CREATE_GROUP_FOLDERS
+        # Store one line responses for filejoin/par2/unrar/unzip here for history display
+        self.action_line = ''
+        # Store the results from various filejoin/par2/unrar/unzip stages
+        self.unpack_info = {}
+        # Stores one line containing the last failure
+        self.fail_msg = ''
+        # Stores various info about the nzo to be
+        if nzo_info:
+            self.nzo_info = nzo_info
+        else:
+            self.nzo_info = {}
 
-        match_result = re.search(NZBFN_MATCHER, filename)
-        if match_result:
-            self.__dirname = match_result.group(1)
+        # Temporary store for custom foldername - needs to be stored because of url/newzbin fetching
+        self.extra1 = nzbname
+
+        self.extra2 = password
+        self.extra3 = None # Will hold earliest next save time of NZO
+        self.extra4 = None # Will hold save timeout for this NZO
+        self.extra5 = None
+        self.extra6 = None
+
+        self.create_group_folder = cfg.CREATE_GROUP_FOLDERS.get()
 
         # Remove leading msgid_XXXX and trailing .nzb
-        self.__dirname, msgid = SplitFileName(self.__dirname)
-        self.__original_dirname = self.__dirname
-        if sabnzbd.REPLACE_SPACES:
-            self.__dirname = self.__dirname.replace(' ','_')
-            self.__original_dirname = self.__dirname
-            logging.info('[%s] Replacing spaces with underscores in %s', __NAME__, self.__dirname)
+        self.__dirname, self.__msgid = SplitFileName(self.__dirname)
+        if msgid:
+            self.__msgid = msgid
 
         if not nzb:
             # This is a slot for a future NZB, ready now
             return
 
-        try:
-            root = XML(nzb)
-        except:
-            logging.warning("[%s] Incorrect NZB file %s (trying anyway)", __NAME__, filename)
+        # Apply conversion option to final folder, called __original_dirname
+        # Yeah, I know :(
+        if cfg.REPLACE_DOTS.get() and ' ' not in self.__original_dirname:
+            logging.info('Replacing dots with spaces in %s', self.__original_dirname)
+            self.__original_dirname = self.__original_dirname.replace('.',' ')
+        if cfg.REPLACE_SPACES.get():
+            logging.info('Replacing spaces with underscores in %s', self.__dirname)
+            self.__original_dirname = self.__original_dirname.replace(' ','_')
 
-        try:
-            root
-        except:
-            logging.error("[%s] Invalid NZB file %s, skipping", __NAME__, filename)
-            raise ValueError
-
-        if not sabnzbd.backup_nzb(filename, nzb, sabnzbd.NO_DUPES):
+        if sabnzbd.backup_exists(filename):
             # File already exists and we have no_dupes set
-            logging.warning('[%s] Skipping duplicate NZB "%s"', __NAME__, filename)
+            logging.warning(Ta('warn-skipDup@1'), filename)
             raise TypeError
 
-        avg_age = 0
-        valids = 0
-        found = 0
-        skipped_files = 0
-        for _file in root:
-            if not self.__group:
-                groups = _file.find('%sgroups' % DTD)
-                if not groups:
-                    groups = _file.find('groups')
-                self.__group = groups[0].text
-            subject = _file.get('subject')
+        # Must create a lower level XML parser because we must
+        # disable the reading of the DTD file from newzbin.com
+        # by setting "feature_external_ges" to 0.
 
-            if isinstance(subject, unicode):
-                subject = subject.encode('latin-1', 'replace')
+        handler = NzbParser(self)
+        parser = xml.sax.make_parser()
+        parser.setFeature(xml.sax.handler.feature_external_ges, 0)
+        parser.setContentHandler(handler)
+        parser.setErrorHandler(xml.sax.handler.ErrorHandler())
+        inpsrc = xml.sax.xmlreader.InputSource()
+        inpsrc.setByteStream(StringIO(nzb))
+        try:
+            parser.parse(inpsrc)
+        except xml.sax.SAXParseException, err:
+            handler.remove_files()
+            logging.warning(Ta('warn-badNZB@3'),
+                          filename, err.getMessage(), err.getLineNumber())
+            raise ValueError
+        except Exception, err:
+            handler.remove_files()
+            logging.warning(Ta('warn-badNZB@3'), filename, err, 0)
+            raise ValueError
 
-            try:
-                t = int(_file.get('date'))
-            except:
-                # NZB has non-standard timestamp, assume 1
-                t = 1
-            date = datetime.datetime.fromtimestamp(t)
+        sabnzbd.backup_nzb(filename, nzb)
 
-            segments = _file.find('%ssegments' % DTD)
-            if not segments:
-                segments = _file.find('segments')
-            nzf = NzbFile(date, subject, segments, self)
+        if cat is None:
+            for grp in self.__group:
+                cat = CatConvert(grp)
+                if cat:
+                    break
 
-            if nzf.valid and nzf.nzf_id:
-                found = 0
-                fln = None
-                if sabnzbd.IGNORE_SAMPLES:
-                    if (nzf.get_filename()):
-                        fln = nzf.get_filename()
-                        fln = fln.lower()
-                    else:
-                        fln = nzf.get_subject()
-                        fln = fln.lower()
+        # Determine category and find pp/script values
+        self.__cat, pp, self.__script, self.__priority = cat_to_opts(cat, pp, script, self.__priority)
+        self.__repair, self.__unpack, self.__delete = sabnzbd.pp_to_opts(pp)
 
-                    sample = re.compile('(^|[\W_])sample\d*[\W_]')
-                    found = sample.search(fln)
+        if cfg.CREATE_GROUP_FOLDERS.get():
+            self.__dirprefix.append(self.get_group())
 
-                if found:
-                    logging.debug("[%s] Sample file found in file: %s, removing: %s." % (__NAME__,fln,nzf.nzf_id))
-                    sabnzbd.remove_data(nzf.nzf_id)
-                else:
-                    logging.info('[%s] %s added to queue', __NAME__, subject)
-                    avg_age += t
-                    valids += 1
-                    self.__files_table[nzf.nzf_id] = nzf
-                    self.__bytes += nzf.bytes()
-                    self.__files.append(nzf)
-
-            else:
-                logging.info('[%s] Error importing %s, skipping', __NAME__,
-                             subject)
-                if nzf.nzf_id:
-                    sabnzbd.remove_data(nzf.nzf_id)
-                skipped_files += 1
-
-        if skipped_files:
-            logging.warning('[%s] Failed to import %s files from %s', __NAME__,
-                             skipped_files, filename)
-
-        if self.__cat == None:
-            self.__cat = CatConvert(self.__group)
-
-        #if sabnzbd.CREATE_GROUP_FOLDERS:
-        self.__dirprefix.append(self.__group)
-
-        #if sabnzbd.CREATE_CAT_FOLDERS and cat:
-        #    self.__dirprefix.append(cat)
-
-        self.__avg_date = datetime.datetime.fromtimestamp(avg_age / valids)
-
-
-        if sabnzbd.AUTO_SORT:
+        if cfg.AUTO_SORT.get():
             self.__files.sort(cmp=_nzf_cmp_date)
         else:
             self.__files.sort(cmp=_nzf_cmp_name)
 
+        # Set nzo save-delay to 6 sec per GB with a max of 5 min
+        self.extra4 = min(6.0 * float(self.__bytes) / GIGI, 300.0)
+
     ## begin nzo.Mutators #####################################################
     ## excluding nzo.__try_list ###############################################
     def check_for_dupe(self, nzf):
@@ -506,6 +625,11 @@ class NzbObject(TryList):
             nzf.deleted = True
         return not bool(self.__files)
 
+    def reset_all_try_lists(self):
+        for nzf in self.__files:
+            nzf.reset_all_try_lists()
+        self.reset_try_list()
+
     def remove_article(self, article):
         nzf = article.nzf
         file_done, reset = nzf.remove_article(article)
@@ -565,7 +689,7 @@ class NzbObject(TryList):
         if not self.__files:
             post_done = True
             #set the nzo status to return "Queued"
-            self.set_status("Queued")
+            self.set_status('Queued')
             self.set_download_report()
 
         return (file_done, post_done, reset)
@@ -589,6 +713,52 @@ class NzbObject(TryList):
     def get_original_dirname(self):
         return self.__original_dirname
 
+    def show_dir_name(self):
+        if self.extra2:
+            return '%s / %s' % (self.__original_dirname, self.extra2)
+        else:
+            return self.__original_dirname
+
+    def get_password(self):
+        return self.extra2
+
+    def set_original_dirname(self, name):
+        if isinstance(name, str):
+            self.__original_dirname = name.strip()
+        else:
+            self.__original_dirname = name
+
+    def set_name(self, name):
+        if isinstance(name, str):
+            name, self.extra2 = scan_password(name)
+            self.__original_dirname = sabnzbd.misc.sanitize_foldername(name)
+            return True
+        return False
+
+    def pause_nzo(self):
+        try:
+            self.__status = 'Paused'
+        except:
+            pass
+
+    def resume_nzo(self):
+        try:
+            self.__status = 'Queued'
+        except:
+            pass
+
+    def get_priority(self):
+        return self.__priority
+
+    def set_priority(self, priority):
+        try:
+            self.__priority = priority
+        except:
+            pass
+
+    def get_msgid(self):
+        return self.__msgid
+
     def add_parfile(self, parfile):
         self.__files.append(parfile)
         parfile.remove_extrapar(parfile)
@@ -596,20 +766,20 @@ class NzbObject(TryList):
     def remove_parset(self, setname):
         self.__partable.pop(setname)
 
-    def set_unpackstr(self, msg, action, stage):
-        if stage not in self.__unpackstrht:
-            self.__unpackstrht[stage] = {}
-        self.__unpackstrht[stage][action] = msg
-
     def set_status(self, status):
         #sets a string outputting the current status of the job, eg:
         #Queued, Downloading, Repairing, Unpacking, Failed, Complete
         self.__status = status
 
     def get_status(self):
-        #returns a string of the current history status
+        #returns a string of the current queued status
         return self.__status
 
+    def get_nzo_id(self):
+        return self.nzo_id
+
+    def get_files(self):
+        return self.__finished_files
 
     def set_download_report(self):
         if self.__avg_bps_total and self.__bytes_downloaded and self.__avg_bps_freq:
@@ -620,32 +790,15 @@ class NzbObject(TryList):
             seconds = timecompleted.seconds
             #find the total time including days
             totaltime = (timecompleted.days/86400) + seconds
+            self.set_nzo_info('download_time',totaltime)
 
             #format the total time the download took, in days, hours, and minutes, or seconds.
-            completestr = ''
-            if timecompleted.days:
-                completestr += '%s day%s ' % (timecompleted.days, self.s_returner(timecompleted.days))
-            if (seconds/3600) >= 1:
-                completestr += '%s hour%s ' % (seconds/3600, self.s_returner((seconds/3600)))
-                seconds -= (seconds/3600)*3600
-            if (seconds/60) >= 1:
-                completestr += '%s minute%s ' % (seconds/60, self.s_returner((seconds/60)))
-                seconds -= (seconds/60)*60
-            if seconds > 0:
-                completestr += '%s second%s ' % (seconds, self.s_returner(seconds))
-            #message 1 - total time
-            completemsg = '%s' % (completestr)
-            self.set_unpackstr(completemsg, '[Time-Taken]', 0)
-            #message 2 - average speed
-            completemsg = '%0.fkB/s' % (avg_bps)
-            self.set_unpackstr(completemsg, '[Avg-Speed]', 0)
+            complete_time = format_time_string(seconds, timecompleted.days)
+
+            self.set_unpack_info('Download', T('msg-DownloadedIn@2') %
+                                 (complete_time, to_units(avg_bps*1024)), unique=True)
 
 
-    def s_returner(self, value):
-        if value > 1:
-            return 's'
-        else:
-            return ''
 
     def get_article(self, server):
         article = None
@@ -658,7 +811,7 @@ class NzbObject(TryList):
                     nzf.finish_import()
                     # Still not finished? Something went wrong...
                     if not nzf.import_finished:
-                        logging.error("[%s] Error importing %s", __NAME__, nzf)
+                        logging.error(Ta('error-qImport@1'), nzf)
                         nzf_remove_list.append(nzf)
                         continue
 
@@ -763,6 +916,9 @@ class NzbObject(TryList):
     def get_dirname(self):
         return self.__dirname
 
+    def get_dirname_rename(self):
+        return self.extra1
+
     def get_dirname_created(self):
         return self.__created
 
@@ -776,11 +932,12 @@ class NzbObject(TryList):
     #        return ''
 
     def get_group(self):
-        return self.__group
+        if self.__group:
+            return self.__group[0]
+        else:
+            return None
 
     def purge_data(self):
-        nzf_ids = []
-
         for nzf in self.__files:
             sabnzbd.remove_data(nzf.nzf_id)
 
@@ -852,23 +1009,70 @@ class NzbObject(TryList):
             avg_date = time.mktime(avg_date.timetuple())
 
         return (self.__repair, self.__unpack, self.__delete, self.__script,
-                self.nzo_id, self.__filename, self.__unpackstrht.copy(),
+                self.nzo_id, self.show_dir_name(), {},
                 self.__msgid, self.__cat, self.__url,
                 bytes_left_all, self.__bytes, avg_date,
-                finished_files, active_files, queued_files, self.__status)
+                finished_files, active_files, queued_files, self.__status, self.__priority)
 
     def get_nzf_by_id(self, nzf_id):
         if nzf_id in self.__files_table:
             return self.__files_table[nzf_id]
 
-    def get_unpackstrht(self):
-        return self.__unpackstrht.copy()
+    def set_unpack_info(self, key, msg, set='', unique=False):
+        '''
+            Builds a dictionary containing the stage name (key) and a message
+            If set is present, it will overwrite any other messages from the set of the same stage
+            If unique is present, it will only have a single line message
+        '''
+        found = False
+        # Unique messages allow only one line per stage(key)
+        if not unique:
+            if not self.unpack_info.has_key(key):
+                self.unpack_info[key] = []
+            # If set is present, look for previous message from that set and replace
+            if set:
+                set = unicoder('[%s]' % set)
+                for x in xrange(len(self.unpack_info[key])):
+                    if set in self.unpack_info[key][x]:
+                        self.unpack_info[key][x] = msg
+                        found = True
+            if not found:
+                self.unpack_info[key].append(msg)
+        else:
+            self.unpack_info[key] = [msg]
+
+    def get_unpack_info(self):
+        return self.unpack_info
+
+    def set_action_line(self, action, msg):
+        if action and msg:
+            self.action_line = '%s: %s' % (action, msg)
+        else:
+            self.action_line = ''
+
+    def get_action_line(self):
+        return self.action_line
+
+    def set_fail_msg(self, msg):
+        self.fail_msg = msg
+
+    def get_fail_msg(self):
+        return self.fail_msg
+
+    def set_nzo_info(self, key, value):
+        self.nzo_info[key] = value
+
+    def get_nzo_info(self):
+        return self.nzo_info
+
+    def set_db_info(self, key, msg):
+        self.nzo_info[key] = msg
 
     def get_repair_opts(self):
         return self.__repair, self.__unpack, self.__delete
 
     def get_pp(self):
-        if self.__repair == None:
+        if self.__repair is None:
             return None
         else:
             return sabnzbd.opts_to_pp(self.__repair, self.__unpack, self.__delete)
@@ -882,6 +1086,15 @@ class NzbObject(TryList):
     def get_future(self):
         return self.__url
 
+    def get_md5pack(self, name):
+        try:
+            return self.md5packs[name]
+        except:
+            return None
+
+    def set_md5pack(self, name, pack):
+        self.md5packs[name] = pack
+
     def __build_pos_nzf_table(self, nzf_ids):
         pos_nzf_table = {}
         for nzf_id in nzf_ids:
@@ -914,68 +1127,47 @@ class NzbObject(TryList):
 
 #-------------------------------------------------------------------------------
 
+def _nzf_get_filename(nzf):
+    # Return filename, if the filename not set, try the
+    # the full subject line instead. Can produce non-ideal results
+    name = nzf.get_filename()
+    if name:
+        return name.lower()
+    else:
+        return nzf.get_subject().lower()
+
+
 def _nzf_cmp_date(nzf1, nzf2):
-    subject1 = nzf1.get_subject().lower()
-    subject2 = nzf2.get_subject().lower()
+    # Compare files based on date, but give vol-par files preference
+    return _nzf_cmp_name(nzf1, nzf2, name=False)
 
-    par2_found = 0
-    ret = 0
-    if 'vol' in subject1 and '.par2' in subject1:
-        par2_found += 1
-        ret -= 1
 
-    if 'vol' in subject2 and '.par2' in subject2:
-        par2_found += 1
-        ret += 1
+def _nzf_cmp_name(nzf1, nzf2, name=True):
+    # The comparison will sort .par2 files to the top of the queue followed by .rar files,
+    # they will then be sorted by name.
+    name1 = _nzf_get_filename(nzf1)
+    name2 = _nzf_get_filename(nzf2)
 
-    if '.rar' in subject1 and not '.par' in subject2 and not '.rar' in subject2: #some nzbs dont get filename field populated, using subject instead
-        return -1 #nzf1 contained '.rar' nzf2 didnt. Move nzf1 up in the queue
+    is_par1 = 'vol' in name1 and '.par2' in name1
+    is_par2 = 'vol' in name2 and '.par2' in name2
+    if is_par1 and not is_par2:
+        return -1
+    if is_par2 and not is_par1:
+        return 1
 
-    if par2_found == 1:
-        return ret
+    if name:
+        # Prioritise .rar files above any other type of file (other than vol-par)
+        # Useful for nzb streaming
+        if  '.rar' in name1 and not is_par2 and '.rar' not in name2:
+            return -1
+        elif '.rar' in name2 and not is_par1 and '.rar' not in name1:
+            return 1
+
+        return cmp(name1, name2)
     else:
+        # Do date comparision
         return cmp(nzf1.get_date(), nzf2.get_date())
 
-def _nzf_cmp_name(nzf1, nzf2):
-    '''
-    The comparison will sort .par2 files to the top of the queue followed by .rar files,
-    they will then be sorted by name.
-    '''
-    #Try to use the filename if it can be extracted from the subject
-    subject1 = nzf1.get_filename()
-    subject2 = nzf2.get_filename()
-
-    #if the filename cannot be extracted, use the full subject line for comparison. Can produce non-ideal results
-    if subject1:
-        subject1 = subject1.lower()
-    else:
-        subject1 = nzf1.get_subject().lower()
-
-    if subject2:
-        subject2 = subject2.lower()
-    else:
-        subject2 = nzf2.get_subject().lower()
-
-    par2_found = 0
-    ret = 0
-    if 'vol' in subject1 and '.par2' in subject1:
-        par2_found += 1
-        ret -= 1
-
-    if 'vol' in subject2 and '.par2' in subject2:
-        par2_found += 1
-        ret += 1
-
-    #Prioritise .rar files above any other type of file (other than .par)
-    #Useful for nzb streaming
-    if '.rar' in subject1 and not '.par' in subject2 and not '.rar' in subject2: #some nzbs dont get filename field populated, using subject instead
-        return -1 #nzf1 contained '.rar' nzf2 didnt. Move nzf1 up in the queue
-
-    if par2_found == 1:
-        return ret
-    else:
-        return cmp(subject1, subject2)
-
 #-------------------------------------------------------------------------------
 
 ################################################################################
@@ -1008,19 +1200,20 @@ def CatConvert(cat):
     found = False
 
     if cat and cat.lower() != 'none':
-        for ucat in sabnzbd.CFG['categories']:
+        cats = config.get_categories()
+        for ucat in cats:
             try:
-                newzbin = sabnzbd.CFG['categories'][ucat]['newzbin']
+                newzbin = cats[ucat].newzbin.get()
                 if type(newzbin) != type([]):
-                    newzbin = listquote.simplelist(newzbin)
+                    newzbin = [newzbin]
             except:
                 newzbin = []
             for name in newzbin:
                 if name.lower() == cat.lower():
-                    if name.find('.') < 0:
-                        logging.debug('[%s] Convert newzbin-cat "%s" to user-cat "%s"', __NAME__, cat, ucat)
+                    if '.' not in name:
+                        logging.debug('Convert newzbin-cat "%s" to user-cat "%s"', cat, ucat)
                     else:
-                        logging.debug('[%s] Convert group "%s" to user-cat "%s"', __NAME__, cat, ucat)
+                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat)
                     newcat = ucat
                     found = True
                     break
@@ -1028,7 +1221,7 @@ def CatConvert(cat):
                 break
 
         if not found:
-            for ucat in sabnzbd.CFG['categories']:
+            for ucat in cats:
                 if cat.lower() == ucat.lower():
                     found = True
                     break
@@ -1038,3 +1231,46 @@ def CatConvert(cat):
     else:
         return None
 
+def format_time_string(seconds, days=0):
+
+    try:
+        seconds = int(seconds)
+    except:
+        seconds = 0
+
+    completestr = ''
+    if days:
+        completestr += '%s day%s ' % (days, s_returner(days))
+    if (seconds/3600) >= 1:
+        completestr += '%s hour%s ' % (seconds/3600, s_returner((seconds/3600)))
+        seconds -= (seconds/3600)*3600
+    if (seconds/60) >= 1:
+        completestr += '%s minute%s ' % (seconds/60, s_returner((seconds/60)))
+        seconds -= (seconds/60)*60
+    if seconds > 0:
+        completestr += '%s second%s ' % (seconds, s_returner(seconds))
+
+    return completestr.strip()
+
+def s_returner(value):
+    if value > 1:
+        return 's'
+    else:
+        return ''
+
+
+RE_PASSWORD1 = re.compile(r'([^/\\]+)[/\\](.+)')
+RE_PASSWORD2 = re.compile(r'(.+){{([^{}]+)}}$')
+def scan_password(name):
+    """ Get password (if any) from the title
+    """
+    if 'http://' in name or 'https://' in name:
+        return name, None
+
+    m = RE_PASSWORD1.search(name)
+    if not m:
+        m = RE_PASSWORD2.search(name)
+    if m:
+        return m.group(1).strip('. '), m.group(2).strip()
+    else:
+        return name.strip('. '), None
